<?php

/**
 * Adds reCaptcha to registration form
 * 
 * @author PrestaPros.com <contact@prestapros.com>
 */

if (!defined('_PS_VERSION_'))
	exit;

class PrestaProsRegisterRecaptcha extends Module
{

    private $_html = '';
    
	public function __construct()
	{
            $this->name = 'prestaprosregisterrecaptcha';
            $this->tab = 'front_office_features';
            $this->version = '1.0';
            $this->author = 'PrestaPros.com';
            $this->need_instance = 0;

            $this->bootstrap = true;

            parent::__construct();

            $this->displayName = $this->l('PrestaPros.com registration reCaptcha');
            $this->description = $this->l('Adds a reCaptcha validation to registration form.');
            $this->confirmUninstall = $this->l('Are you sure you want to delete this module?');
            $this->ps_versions_compliancy = array('min' => '1.6', 'max' => '1.6.99.99');
	}
	
	public function install()
	{
            if (!parent::install() OR
                !$this->registerHook('displayCustomerAccountForm') OR 
                !$this->registerHook('actionBeforeSubmitAccount')
                )
                return false;
            return true;
	}
	
	public function uninstall()
	{
            if (!parent::uninstall() ||
                !$this->_eraseConfig()
                )
                return false;
            return true;
	}

	public function getContent()
	{
            $this->_postProcess();
            $this->_displayForm();

            return $this->_html;
	}
	
	private function _eraseConfig()
	{
                Configuration::deleteByName('PRESTAPROS_REGISTER_RECAPTCHA_SITE_KEY');
                Configuration::deleteByName('PRESTAPROS_REGISTER_RECAPTCHA_SECRET_KEY');
		return true;
	}        
        
	private function _displayForm()
	{
            $this->_html .= $this->_generateForm();
            $this->_html .= '<div class="bootstrap panel">Created by: <a href="https://prestapros.com/" target="_blank">PrestaPros.com</a></div>';
	}

	private function _generateForm()
	{
		$inputs = array();

		$inputs[] = array(
			'type' => 'text',
			'label' => $this->l('Google reCaptcha site key'),
			'name' => 'PRESTAPROS_REGISTER_RECAPTCHA_SITE_KEY',
			'desc' => 'Obtain keys https://www.google.com/recaptcha/admin/create. Tested on reCaptcha v2.'
			);

		$inputs[] = array(
			'type' => 'text',
			'label' => $this->l('Google reCaptcha secret key'),
			'name' => 'PRESTAPROS_REGISTER_RECAPTCHA_SECRET_KEY',
			'desc' => 'Obtain keys https://www.google.com/recaptcha/admin/create. Tested on reCaptcha v2.'
			);
                

		$fields_form = array(
			'form' => array(
				'legend' => array(
					'title' => $this->l('Settings'),
					'icon' => 'icon-cogs'
					),
				'input' => $inputs,
				'submit' => array(
					'title' => $this->l('Save'),
					'class' => 'btn btn-default pull-right',
					'name' => 'submitUpdate'
					)
				)
			);



		$lang = new Language((int)Configuration::get('PS_LANG_DEFAULT'));
		$helper = new HelperForm();
		$helper->default_form_language = $lang->id;
		$helper->currentIndex = $this->context->link->getAdminLink('AdminModules',false).'&configure='.$this->name.'&tab_module='.$this->tab.'&module_name='.$this->name;
		$helper->token = Tools::getAdminTokenLite('AdminModules');
		$helper->tpl_vars = array(
			'fields_value' => array(
                            'PRESTAPROS_REGISTER_RECAPTCHA_SITE_KEY' => Configuration::get('PRESTAPROS_REGISTER_RECAPTCHA_SITE_KEY'),
                            'PRESTAPROS_REGISTER_RECAPTCHA_SECRET_KEY' => Configuration::get('PRESTAPROS_REGISTER_RECAPTCHA_SECRET_KEY')
                        ),
			'languages' => $this->context->controller->getLanguages(),
			'id_language' => $this->context->language->id
		);
		return $helper->generateForm(array($fields_form));
	}


	private function _postProcess()
	{
            if (Tools::isSubmit('submitUpdate'))
            {
                $site_key = Tools::getValue('PRESTAPROS_REGISTER_RECAPTCHA_SITE_KEY');
                $secret_key = Tools::getValue('PRESTAPROS_REGISTER_RECAPTCHA_SECRET_KEY');
                if (empty($site_key) || empty($secret_key)) {
                    $this->_html .= $this->displayError($this->l('Both keys are required'));
                } else {
                    Configuration::updateValue('PRESTAPROS_REGISTER_RECAPTCHA_SITE_KEY', pSQL($site_key));
                    Configuration::updateValue('PRESTAPROS_REGISTER_RECAPTCHA_SECRET_KEY', pSQL($secret_key));
                    $this->_html .= $this->displayConfirmation($this->l('Settings Updated!'));
                }
            }
	}

	public function hookDisplayCustomerAccountForm($params)
	{
            if (!empty(Configuration::get('PRESTAPROS_REGISTER_RECAPTCHA_SITE_KEY')) 
                    && !empty(Configuration::get('PRESTAPROS_REGISTER_RECAPTCHA_SECRET_KEY'))) {
                return '<div class="row"><div class="col-xs-12 col-sm-12" style="margin-bottom: 10px; padding-left: 5px;"><div class="g-recaptcha" data-sitekey="'.Configuration::get('PRESTAPROS_REGISTER_RECAPTCHA_SITE_KEY').'"></div></div></div><script src=\'https://www.google.com/recaptcha/api.js\'></script>';
            }
	}
	
	public function hookActionBeforeSubmitAccount($params)
	{
            if (!empty(Configuration::get('PRESTAPROS_REGISTER_RECAPTCHA_SITE_KEY')) && !empty(Configuration::get('PRESTAPROS_REGISTER_RECAPTCHA_SECRET_KEY'))) {
                if (!isset($_POST['g-recaptcha-response']) || empty($_POST['g-recaptcha-response'])) {
                    $this->context->controller->errors[] = $this->l('Please use recaptcha');
                    return;
                }
                
                //captcha verification
                $googleVerificationUrl = 'https://www.google.com/recaptcha/api/siteverify';
                $googleSec = Configuration::get('PRESTAPROS_REGISTER_RECAPTCHA_SECRET_KEY');

                $ch = curl_init();
    
                curl_setopt_array($ch, [
                    CURLOPT_URL => $googleVerificationUrl,
                    CURLOPT_POST => true,
                    CURLOPT_POSTFIELDS => [
                        'secret' => $googleSec,
                        'response' => $_POST['g-recaptcha-response'],
                        'remoteip' => $_SERVER['REMOTE_ADDR']
                    ],
                    CURLOPT_RETURNTRANSFER => true
                ]);
    
                $output = curl_exec($ch);
                curl_close($ch);            
                
                $googleResponseObj = json_decode($output);
                if($googleResponseObj->success !== true)
                {
                    $this->context->controller->errors[] = $this->l('reCaptcha verification failed');
                    return;
                } else {
                    return true;
                }                
            }            
	}
}
